; ---------------------------------------------------------------------------------------------
; Teclado matricial simples (matrix 4x4)                                             03/12/2020
;
; STATUS: FUNCIONA
;
; Leitura basica de um teclado simples de matrix 4x4 exibindo a tecla pressionada em 8 LEDS
; conectados ao PORTC.
;
; Este projeto foi simulado com o esquema desenhado no Proteus que esta na pasta
; "\Simulacao_Proteus\kbd4x4.DSN" deste projeto.
;
; O simbolo KEYPORT pode representar qualquer porta do microcontrolador que possa fornecer
; 8 bits completos. Neste programa o PORTB foi usada mas o programador pode escolher outras.
;
; Na porta de I/O, representada por KEYPORT, os bits 0 a 3 serao definidos como saidas e
; receberao as linhas do teclado. Os bits de 4 a 7 receberao as colunas, serao definidos como 
; entradas e deverao ser mantidos em nivel alto com resistores Pull Up.
;
; A estrategia de leitura consiste em desligar um bit de saida (0 a 3) e testar qual bit de 
; entrada (4 a 7) foi desligado, descobrindo-se assim o caracter digitado conforme o mapeamento
; das teclas abaixo:
; ---------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------
; MAPEAMENTO DAS TECLAS
; ---------------------------------------------------------------------------------------------
;         ENTRADA (bits)     SAIDA (bits)
;         -------------      -------------
; KEYPORT 7   6   5   4      3   2   1   0
; ----------------------     ------------- 
; teclas                         
;   1     0   1   1   1      1   1   1   0  LINHA 1 X COLUNA 1
;   2     1   0   1   1      1   1   1   0  LINHA 1 X COLUNA 2
;   3     1   1   0   1      1   1   1   0  LINHA 1 X COLUNA 3
;   A     1   1   1   0      1   1   1   0  LINHA 1 X COLUNA 4
;  
;   4     0   1   1   1      1   1   0   1  LINHA 2 X COLUNA 1
;   5     1   0   1   1      1   1   0   1  LINHA 2 X COLUNA 2
;   6     1   1   0   1      1   1   0   1  LINHA 2 X COLUNA 3
;   B     1   1   1   0      1   1   0   1  LINHA 2 X COLUNA 4
; 
;   7     0   1   1   1      1   0   1   1  LINHA 3 X COLUNA 1
;   8     1   0   1   1      1   0   1   1  LINHA 3 X COLUNA 2
;   9     1   1   0   1      1   0   1   1  LINHA 3 X COLUNA 3                           
;   C     1   1   1   0      1   0   1   1  LINHA 3 X COLUNA 4
;
;   *     0   1   1   1      0   1   1   1  LINHA 4 X COLUNA 1
;   0     1   0   1   1      0   1   1   1  LINHA 4 X COLUNA 2
;   #     1   1   0   1      0   1   1   1  LINHA 4 X COLUNA 3
;   D     1   1   1   0      0   1   1   1  LINHA 4 X COLUNA 4                                           
;
; ---------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------
; HARDWARE
; Fosc ................ 4MHZ interno
; KEYPORT<0> .......... Saida   - linha  1 do teclado
; KEYPORT<1> .......... Saida   - linha  2 do teclado
; KEYPORT<2> .......... Saida   - linha  3 do teclado
; KEYPORT<3> .......... Saida   - linha  4 do teclado
; KEYPORT<4> .......... Entrada - coluna 4 do teclado (pull up 10K)
; KEYPORT<5> .......... Entrada - coluna 3 do teclado (pull up 10K)
; KEYPORT<6> .......... Entrada - coluna 2 do teclado (pull up 10K)
; KEYPORT<7> .......... Entrada - coluna 1 do teclado (pull up 10K)
; PORTC<7:0> .......... LEDS
;
; ---------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------
; ESQUEMA PARA CONEXAO DE CADA TECLA
; ---------------------------------------------------------------------------------------------
;                  VCC
;                   |  
;                   |      
;                   |   
;                  +-+ 
;              10K | |           
;                  | |    
;                  +-+       TECLA     
;                   |          | 
;                   |         ===   
; KEYPORT<4-7>------0---------+ +------- KEYPORT<0-3> 
;
; ---------------------------------------------------------------------------------------------

	LIST P=18F4520
	#include <P18F4520.INC>
    #include "familias.inc"
;
; ---------------------------------------------------------------------------------------------
; CONFIGURATION BITS ou FUSES
; ---------------------------------------------------------------------------------------------
	    CONFIG MCLRE  = ON
        CONFIG OSC    = INTIO67

; ---------------------------------------------------------------------------------------------
; REGISTRADORES E SIMBOLOS NA MEMORIA DE DADOS
; ---------------------------------------------------------------------------------------------
             UDATA  0x80
CLOCK        equ    .4000000
delay_temp   res    1
R1           res    1              ; registrador de trabalho

#define        KEYTRIS TRISB       ; direcao de I/O dos pinos do teclado
#define        KEYPORT PORTB       ; porta de I/O do teclado
#define        KEYLAT  LATB        ; latch do teclado

rstvec     CODE    0x0000
           goto    inicio
inth       CODE    0x0008
           retfie


MAIN       CODE
           #include "delay_services.inc"

inicio     movlw    0x0F
           movwf    ADCON1

           clrf     LATA
           clrf     LATB
           clrf     LATC
           clrf     LATD
           clrf     KEYLAT

           movlw    0x00
           movwf    TRISA
           movwf    TRISB
           movwf    TRISC
           movwf    TRISD
           movwf    TRISE

           movlw    0xF0           ; KEYPORT<7:4> --> entradas
                                   ; KEYPORT<3:0> --> saidas  
           movwf    KEYTRIS        ; estabelece direcoes dos pinos de I/O

; -----------------------------------------------------------------
; Rotina principal
; -----------------------------------------------------------------
loop       call    getKbdChar      ; vai ler o teclado (W contera o resultado)
           movwf   LATC            ; mostra W nos LEDs
           goto    loop            ; permanece neste loop

; -----------------------------------------------------------------
; Funcao 'getKbdChar'
; Le um caracter no teclado matricial e retorna a tecla pressionada
; no registrador W.
;
; Entrada: Nao ha
; Saida  : W contem o caracter lido
;------------------------------------------------------------------
getKbdChar 
           ; Verificacao da linha 1 --> KEYLAT<0>
           call     latchClear
           bcf      KEYLAT, 0                    ; varredura linha 1
           movlw    "1"
           btfss    KEYPORT, 7                   ; tecla "1" ?
           goto     debounce                     ; SIM -->
           movlw    "2"
           btfss    KEYPORT, 6                   ; tecla "2" ?
           goto     debounce                     ; SIM -->
           movlw    "3"
           btfss    KEYPORT, 5                   ; tecla "3" ?
           goto     debounce                     ; SIM -->
           movlw    "A"
           btfss    KEYPORT, 4                   ; tecla "A" ?
           goto     debounce                     ; SIM -->

           ; Verificacao da linha 2 --> KEYLAT<1>
           call     latchClear
           bcf      KEYLAT, 1                    ; varredura linha 2
           movlw    "4"
           btfss    KEYPORT, 7                   ; tecla "4" ?
           goto     debounce                     ; SIM -->
           movlw    "5"
           btfss    KEYPORT, 6                   ; tecla "5" ?
           goto     debounce                     ; SIM -->
           movlw    "6"
           btfss    KEYPORT, 5                   ; tecla "6" ?
           goto     debounce                     ; SIM -->
           movlw    "B"
           btfss    KEYPORT, 4                   ; tecla "B" ?
           goto     debounce                     ; SIM -->

           ; Verificacao da linha 3 --> KEYLAT<2>
           call     latchClear
           bcf      KEYLAT, 2                    ; varredura linha 3
           movlw    "7"
           btfss    KEYPORT, 7                   ; tecla "7" ?
           goto     debounce                     ; SIM -->
           movlw    "8"
           btfss    KEYPORT, 6                   ; tecla "8" ?
           goto     debounce                     ; SIM -->
           movlw    "9"
           btfss    KEYPORT, 5                   ; tecla "9" ?
           goto     debounce                     ; SIM -->
           movlw    "C"
           btfss    KEYPORT, 4                   ; tecla "C" ?
           goto     debounce                     ; SIM -->

           ; Verificacao da linha 4 --> KEYLAT<3>
           call     latchClear
           bcf      KEYLAT, 3                    ; varredura linha 4
           movlw    "*"
           btfss    KEYPORT, 7                   ; tecla "*" ?
           goto     debounce                     ; SIM -->
           movlw    "0"
           btfss    KEYPORT, 6                   ; tecla "0" ?
           goto     debounce                     ; SIM -->
           movlw    "#"
           btfss    KEYPORT, 5                   ; tecla "#" ?
           goto     debounce                     ; SIM -->
           movlw    "D"
           btfss    KEYPORT, 4                   ; tecla "D" ?
           goto     debounce                     ; SIM -->

           goto     getKbdChar

; -------------------------------------------------------------
; Debounce da tecla pressionada.
; Espera o mano tirar o dedao da tecla pra evitar rebote de
; leitura.
; -------------------------------------------------------------
debounce   movwf    R1                           ; salva a tecla
           movlw    .60                          ; milisegundos
           call     delay_ms                     ; debounce
debounce2  movf     KEYPORT, w                   ; le a porta
           andlw    0xF0                         ; deixa so os bits que interessam 
           xorlw    0xF0                         ; verifica
           btfss    STATUS, Z                    ; tecla liberara?
           goto     debounce2                    ; NAO --> espera liberar
           movf     R1, w                        ; recupera a tecla
           return                                ; fim

; ------------------------------------------------------------
; Limpa o latch
; ------------------------------------------------------------
latchClear movlw    0x0F
           movwf    KEYLAT
           return
;
           END 